@@ -1,9 +1,9 @@
 import typing as T
-from dgraph_orm import Node
+from dgraph_orm import Node as DBNode
 from pydantic import BaseModel, PrivateAttr
 
 StrawberryType = T.TypeVar("StrawberryType")
-NodeType = T.TypeVar("NodeType", bound=Node)
+NodeType = T.TypeVar("NodeType", bound=DBNode)
 PydanticType = T.TypeVar("PydanticType", bound=BaseModel)
 
 
@@ -63,8 +63,8 @@ class Display(T.Generic[NodeType]):
         return self._node
 
 
-DBType = T.TypeVar("DBType", bound=Node)
-LayerNodeType = T.TypeVar("LayerNodeType", bound=Node)
+DBType = T.TypeVar("DBType", bound=DBNode)
+LayerNodeType = T.TypeVar("LayerNodeType", bound=DBNode)
 
 # TODO prob rename this...
 class Node(T.Generic[DBType]):
@@ -74,6 +74,7 @@ class Node(T.Generic[DBType]):
         # but must also transfer private fields and cache!
         n = cls(**model_db.dict())
         # for all private fields too
+        # ooohhh, this is for the cache!
         for private_field in model_db.__private_attributes__.keys():
             setattr(n, private_field, getattr(model_db, private_field))
         return n
