@@ -41,8 +41,8 @@ def imports() -> str:
         "from uuid import UUID",
         "from decimal import Decimal",
         "from edgedb import RelativeDuration, AsyncIOClient, create_async_client",
-        "from pydantic import BaseModel, Field",
-        f"from {PATH_TO_MODULE} import Node, Resolver, NodeException, ResolverException, UpdateOperation, Batch",
+        "from pydantic import BaseModel, Field, PrivateAttr",
+        f"from {PATH_TO_MODULE} import Node, Resolver, NodeException, ResolverException, UpdateOperation, Batch, Unset, ComputedPropertyException, is_unset",
     ]
     return "\n".join(lines)
 
@@ -138,11 +138,21 @@ class {CONFIG_NAME}:
     """
 
 
-def stringify_dict(d: T.Dict[str, str]) -> str:
-    inner = [f'"{k}":"{v}"' for k, v in d.items()]
+def stringify_dict(d: T.Union[T.Dict[str, str], str]) -> str:
+    if type(d) is not dict:
+        s = f"{d}"
+        if type(d) is not bool:
+            s = f'"{s}"'
+        return s
+    inner = [f'"{k}":{stringify_dict(v)}' for k, v in d.items()]
     return f"{{{','.join(inner)}}}"
 
 
+def stringify_set(s: T.Set[str]) -> str:
+    strs: T.List[str] = [f'"{i}"' for i in s]
+    return "{" + ",".join(strs) + "}"
+
+
 def edgedb_conversion_type_from_prop(prop: Property) -> str:
     s = prop.target.name
     pattern = r"default::\w+"
@@ -153,11 +163,11 @@ def edgedb_conversion_type_from_prop(prop: Property) -> str:
 def build_node_and_resolver(object_type: ObjectType) -> str:
     # need to sort props and links by required, exclusive, no default, rest
     object_type.properties.sort(
-        key=lambda x: f"{x.required}-{x.is_exclusive}-{x.default}",
+        key=lambda x: f"{not x.is_computed}-{x.required}-{x.is_exclusive}-{x.default}",
         reverse=True,
     )
     object_type.links.sort(
-        key=lambda x: f"{x.required}-{x.is_exclusive}-{x.default}",
+        key=lambda x: f"{not x.is_computed}-{x.required}-{x.is_exclusive}-{x.default}",
         reverse=True,
     )
     # start with the properties
@@ -167,42 +177,83 @@ def build_node_and_resolver(object_type: ObjectType) -> str:
     updatable_fields: T.Set[str] = set()
     exclusive_fields: T.Set[str] = set()
 
-    node_edgedb_conversion_map: T.Dict[str, str] = {}
-    insert_edgedb_conversion_map: T.Dict[str, str] = {}
+    node_edgedb_conversion_map: T.Dict[str, T.Dict[str, str]] = {}
+    insert_edgedb_conversion_map: T.Dict[str, T.Dict[str, str]] = {}
+
+    computed_properties: T.Set[str] = set()
+    computed_property_getter_strs: T.List[str] = []
 
     for prop in object_type.properties:
         conversion_type = edgedb_conversion_type_from_prop(prop)
-        node_edgedb_conversion_map[prop.name] = conversion_type
-        if not prop.readonly:
+        node_edgedb_conversion_map[prop.name] = {
+            "cast": conversion_type,
+            "cardinality": prop.cardinality.value,
+            "readonly": prop.readonly,
+        }
+        if prop.is_computed:
+            computed_properties.add(prop.name)
+        if not prop.readonly and not prop.is_computed:
             updatable_fields.add(prop.name)
         if prop.is_exclusive:
             exclusive_fields.add(prop.name)
         default_value_str = "..." if prop.required else "None"
-        allow_mutation_str = f"allow_mutation={not prop.readonly}"
-        property_strs.append(
-            f"{prop.name}: {prop.type_str} = Field({default_value_str}, {allow_mutation_str})"
+        allow_mutation_str = (
+            f"allow_mutation={not prop.readonly and not prop.is_computed}"
         )
+        if not prop.is_computed:
+            property_strs.append(
+                f"{prop.name}: {prop.type_str} = Field({default_value_str}, {allow_mutation_str})"
+            )
+        else:
+            property_strs.append(
+                f"_{prop.name}: T.Union[Unset, {prop.type_str}] = PrivateAttr(default_factory=Unset)"
+            )
+            computed_property_getter_strs.append(
+                f"""
+@property
+def {prop.name}(self) -> {prop.type_str}:
+    if is_unset(self._{prop.name}):
+        if "{prop.name}" in self.extra:
+            self._{prop.name} = self.extra["{prop.name}"]
+        else:
+            raise ComputedPropertyException("{prop.name} is unset")
+    return self._{prop.name}
+                """
+            )
         # for insert type
         if prop.name != "id":
-            insert_edgedb_conversion_map[prop.name] = conversion_type
-            insert_type_str = prop.type_str
-            # if required but has default, add optional back
-            if prop.required and prop.default:
-                insert_type_str = f"T.Optional[{insert_type_str}]"
-            default_value_str = (
-                " = None" if insert_type_str.startswith("T.Optional[") else ""
-            )
-            insert_property_strs.append(
-                f"{prop.name}: {insert_type_str}{default_value_str}"
-            )
+            if not prop.is_computed and not prop.not_insertable:
+                insert_edgedb_conversion_map[prop.name] = {
+                    "cast": conversion_type,
+                    "cardinality": prop.cardinality.value,
+                    "readonly": prop.readonly,
+                }
+                insert_type_str = prop.type_str
+                # if required but has default, add optional back
+                if prop.required and prop.default:
+                    insert_type_str = f"T.Optional[{insert_type_str}]"
+                default_value_str = (
+                    " = None" if insert_type_str.startswith("T.Optional[") else ""
+                )
+                insert_property_strs.append(
+                    f"{prop.name}: {insert_type_str}{default_value_str}"
+                )
     link_function_strs: T.List[str] = []
     resolver_function_strs: T.List[str] = []
     updatable_links: T.Set[str] = set()
     exclusive_links: T.Set[str] = set()
+    link_conversion_map: T.Dict[str, T.Dict[str, str]] = {}
+
     for link in object_type.links:
         if link.name == "__type__":
             continue
-        if not link.readonly:
+        link_conversion_map[link.name] = {
+            "cast": link.target.model_name,
+            "cardinality": link.cardinality.value,
+            "readonly": link.readonly,
+            "required": link.required,
+        }
+        if not link.readonly and not link.is_computed:
             updatable_links.add(link.name)
         if link.is_exclusive:
             exclusive_links.add(link.name)
@@ -213,15 +264,16 @@ def build_node_and_resolver(object_type: ObjectType) -> str:
             )
         )
         # for insert
-        insert_resolver_str = f"{link.target.model_name}Resolver"
-        if (not link.required) or (link.required and link.default):
-            insert_resolver_str = f"T.Optional[{insert_resolver_str}]"
-        default_value_str = (
-            " = None" if insert_resolver_str.startswith("T.Optional[") else ""
-        )
-        insert_property_strs.append(
-            f"{link.name}: {insert_resolver_str}{default_value_str}"
-        )
+        if not link.is_computed and not link.not_insertable:
+            insert_resolver_str = f"{link.target.model_name}Resolver"
+            if (not link.required) or (link.required and link.default):
+                insert_resolver_str = f"T.Optional[{insert_resolver_str}]"
+            default_value_str = (
+                " = None" if insert_resolver_str.startswith("T.Optional[") else ""
+            )
+            insert_property_strs.append(
+                f"{link.name}: {insert_resolver_str}{default_value_str}"
+            )
     orm_config_str = build_orm_config(
         model_name=object_type.node_name,
         updatable_fields={*updatable_fields, *updatable_links},
@@ -237,7 +289,10 @@ def build_node_and_resolver(object_type: ObjectType) -> str:
 
     # node
     node_properties_str = "\n".join(property_strs)
+    computed_property_getter_str = "\n".join(computed_property_getter_strs)
     node_conversion_map_str = f"_edgedb_conversion_map: T.ClassVar[T.Dict[str, str]] = {stringify_dict(node_edgedb_conversion_map)}"
+    insert_link_conversion_map_str = f"_link_conversion_map: T.ClassVar[T.Dict[str, str]] = {stringify_dict(link_conversion_map)}"
+    computed_properties_str = f"_computed_properties: T.ClassVar[T.Set[str]] = {stringify_set(computed_properties)}"
     node_link_functions_str = "\n".join(link_function_strs)
     update_function_str = build_update_function_str(
         node_resolver_name=node_resolver_name, links=object_type.links
@@ -245,7 +300,10 @@ def build_node_and_resolver(object_type: ObjectType) -> str:
     node_inner_strs = [
         node_properties_str,
         "\n",
+        computed_property_getter_str,
         node_conversion_map_str,
+        insert_link_conversion_map_str,
+        computed_properties_str,
         node_link_functions_str,
         update_function_str,
         orm_config_str,
