@@ -15,7 +15,9 @@ from fastapi.encoders import jsonable_encoder
 import json
 from pathlib import Path
 
-from dgraph_orm.vendors.edgedb.gen import edgedb_DO as DB
+# from dgraph_orm.vendors.edgedb.gen import edgedb_DO as DB
+
+from dgraph_orm.vendors.edgedb.gen import aws as DB
 
 
 async def test_query():
@@ -348,7 +350,7 @@ async def test_update_links():
 
 async def test_batch():
     batch = Batch(print_status=True, client=DB.client)
-    for i in range(400):
+    for i in range(100):
         insert_person = DB.PersonInsert(
             first_name=f"JonWHOO{i}",
             last_name=f"QuackERS{i}",
@@ -423,7 +425,7 @@ async def test_transaction():
                 debug(p)
 
 
-from dgraph_orm.vendors.edgedb.constants import random_str
+from dgraph_orm.vendors.edgedb.constants import random_str, chunk_list, flatten_list
 
 
 async def update_all_people():
@@ -513,6 +515,9 @@ def people_inserts(n: int) -> T.List[DB.PersonInsert]:
             last_name=f"QuackERS{i}",
             tags={"hey", "yaaas"},
             slug=random_str(6),
+            best_friend=DB.PersonResolver().filter(
+                ".slug = <str>$slug", {"slug": "jeremy"}
+            ),
         )
         for i in range(n)
     ]
@@ -574,6 +579,7 @@ async def test_extra_fields():
         .limit(100)
         .offset(1)
         .extra_fields(
+            "firebase_id := [is ExternalBooking].firebase_id ?? [is Booking].firebase_id, "
             "is_external := exists [is ExternalBooking].id, "
             "artist_name := [is ExternalBooking].artist_name ?? [is Booking].artist.name, "
             "artist_slug := [is Booking].artist.slug, "
@@ -587,7 +593,7 @@ async def test_extra_fields():
     for meta in meta_bookings:
         calendar_bookings.append(
             CalendarBooking(
-                firebase_id=meta.firebase_id,
+                firebase_id=meta.extra.get("firebase_id"),
                 artist_name=meta.extra.get("artist_name"),
                 start_time=meta.start_time,
                 booking_type=CalendarBookingType.external
@@ -668,50 +674,245 @@ select models {{id}}
 
 
 async def using_for():
-    n = 1000
+    n = 10
     query_s = f"""
 with
     raw_data := <json>$data,
 for item in json_array_unpack(raw_data) union (
-    insert Person {{ first_name := <str>item['first_name'], slug := <str>item['slug'], last_name := <str>item['last_name'] }}
-) {{id, slug, last_name, created_at}}
+    with
+        # ordered_tags_var := <array<str>>item['ordered_tags'] if false else <array<str>>$empty_array,
+        # ordered_tags := <OPTIONAL array<str>>json_get(item, 'ordered_tags_miss'),
+        # ordered_tags := <array<str>>json_get(item, 'blo'),
+        # ordered_tags := <array<str>>json_get(item, 'json_miss'),
+        first_name_var := <str>item['first_name'] if exists <str>json_get(item, 'first_name') else <str>"notset",
+        # last_name := <str>item['last_name'] ?? <str>"jon"
+        last_name := <str>json_get(item, 'last_namer'),
+        # best_friend_filter := <str>".first_name = <str>"Jeremy""
+        # best_friend_filter := filter .slug in array_unpack(<array<str>>json_get(item, 'best_friend_slugs')),
+        # filter_name := <str>".slug",
+        best_friend := (select detached Person filter .slug = <str>json_get(item, 'best_friend', 'filter') limit <int16>json_get(item, 'best_friend', 'limit'))
+    insert Person {{ first_name := first_name_var, slug := <str>item['slug'], last_name := last_name, best_friend := best_friend}} unless conflict on .slug else (update Person set {{ first_name := <str>item['first_name'] }})
+) {{id, slug, last_name, ordered_tags, tags, inserted_at, created_at, best_friend: {{first_name}} }}
         """
     data = [
         {
             "first_name": random_str(10),
             "slug": random_str(10),
-            "last_name": "Scooby" if i % 2 == 0 else None,
+            # "slug": "EPwaYEelMB",
+            # "last_name": "Scooby" if i % 2 == 0 else None,
+            "last_name": None,
+            "tags": ["hey"],
+            # "tags": [],
+            # "tags": [],
+            "ordered_tags": ["spooky"],
+            "best_friend_slugs": ["jeremy"],
+            "best_friend": {"slug": "jeremy"},
+            # "ordered_tags": None,
+            # "ordered_tags": [],
         }
         for i in range(n)
     ]
-    variables = {"data": json.dumps(data)}
+    # learning -> if supposed to be a set or list or tuple but None, set to the empty versions of those things
+    # this works if i can treat None as the same as an empty array or set or tuple
+    # OK so NULL === {[], {}, ())... will just have to deal with this
+    variables = {
+        "data": json.dumps(data),
+        # "empty_array": ["hi"],
+    }
     res = await execute_query(client=DB.client, query_str=query_s, variables=variables)
     debug(res)
 
 
-async def main():
-    # await test_get()
-    # await test_nested()
-    # await test_add()
-    # await raw_add_test()
-    # await test_raw_add_many()
-    # await test_raw_insert_without_for_loop()
-    # await test_delete()
-    # await test_update()
-    # await test_link()
-    # await test_update_links()
-    # await test_batch()
-    # await test_refresh()
-    # await test_transaction()
-    # await test_upsert()
-    #
-    # await test_generated_add()
-    # await test_generated_delete()
+async def test_add_many_new():
+    n = 10
+    inserts = [DB.PersonInsert(first_name="jay", slug=random_str(10)) for _ in range(n)]
+    res = await DB.Person.add_many(inserts=inserts, upsert_given_conflict_on="slug")
+    debug(res)
 
-    # await test_extra_fields()
-    # await test_bulk_reads()
-    # await investigate_bulk_load_bug()
+
+from dgraph_orm.vendors.edgedb import creating_strings
+
+
+async def creating_strings_testing():
+    edge_filter_strs = [
+        "best_friend := (select detached Person filter .slug = <str>json_get(item, 'best_friend', 'slug')"
+    ]
+    insert_str = creating_strings.insert_str_from_cls(
+        insert_cls=DB.PersonInsert,
+        node_cls=DB.Person,
+        edge_filter_strs=edge_filter_strs,
+        # upsert_given_conflict_on="slug",
+        return_conflicting_model_on="slug",
+    )
+    # print(insert_str)
+    insert = DB.PersonInsert(
+        first_name="Juanald",
+        slug="juanald",
+        last_name=random_str(10),
+        best_friend=DB.PersonResolver().filter(
+            ".slug = <str>$slug", {"slug": "jeremy"}
+        ),
+    )
+    people = await DB.Person.add_many(
+        inserts=[insert],
+        # upsert_given_conflict_on="slug",
+        return_conflicting_model_on="slug",
+    )
+    debug(people)
+    debug(await people.pop().best_friend())
+
+
+async def testing_with_dog():
+    inserts = [
+        DB.DogInsert(
+            name=f"Spoof{i}wwz",
+            # slug=random_str(10),
+            slug=random_str(10) if i != 0 else "dog",
+            # slug="dog",
+        )
+        for i in range(100)
+    ]
+    dogs = await DB.Dog.add_many(
+        inserts=inserts,
+        # return_conflicting_model_on="slug",
+        upsert_given_conflict_on="slug",
+    )
+    debug(dogs)
+
+
+async def testing_with_dog_raw():
+    working_s = """
+with
+    raw_data := <json>$data,
+for item in <tuple<slug: str, name: str>> json_array_unpack(raw_data) union (
+    insert Dog {slug := item.slug, name := item.name}
+    unless conflict on .slug else (select Dog)
+) { slug, name, id }
+    """
+    testing_s = """
+with
+    raw_data := <json>$data,
+for item in json_array_unpack(raw_data) union (
+    with
+        slug := <str>json_get(item, 'slug'),
+        name := <str>json_get(item, 'name')
+    insert Dog {slug := slug, name := name}
+    # unless conflict on .slug else (select Dog)
+    unless conflict on .slug else (update Dog set {slug := slug, name := name})
+) { slug, name, id }
+        """
+    dogs = await execute_query(
+        client=DB.client,
+        query_str=testing_s,
+        variables={
+            "data": json.dumps(
+                [
+                    {"slug": "dog", "name": "jonny"},
+                    {"slug": "dog00", "name": "jonnyald"},
+                ]
+            )
+        },
+    )
+    debug(dogs)
+
+
+async def testing_with_person():
+    inserts = people_inserts(10_000)
+    # people = await DB.Person.add_many(inserts=inerts)
+    proms = []
+    for chunk in chunk_list(inserts, chunk_size=10000):
+        proms.append(
+            DB.Person.add_many(
+                chunk,
+                edge_filter_strs=[
+                    "best_friend := (select detached Person filter .slug = <str>json_get(item, 'best_friend', 'slug'))"
+                    # "best_friend := (select detached Person order by .inserted_at asc limit 1)"
+                ],
+            )
+        )
+    res = flatten_list(await asyncio.gather(*proms))
+    print(len(res))
+    # print(len(people))
+    if False:
+        from dgraph_orm.vendors.edgedb.gen.edgedb_DO import client
+
+        async for tx in client.transaction():
+            async with tx:
+                for insert in inerts:
+                    p = await DB.Person.add(insert, given_client=tx)
+                    debug(p)
+
+
+async def testing_add_many_with_people():
+    inserts = people_inserts(100)
+    inserts[0].slug = "dog"
+    inserts[1].slug = "dog"
+    # inserts[2].slug = "dog"
+    # inserts[0].slug = "sjYZbHr"
+    people = await DB.Person.add_many(
+        inserts=inserts,
+        # return_conflicting_model_on="slug",
+        upsert_given_conflict_on="slug",
+    )
+    debug(people)
+
+
+from dgraph_orm.vendors.edgedb.unset import ComputedPropertyException
+
+
+async def test_default_computed_fields():
+    person = await DB.PersonResolver().get(slug="jeremy")
+    try:
+        print(person.full_name)
+        assert 1 == 2
+    except Exception as e:
+        assert type(e) is ComputedPropertyException
+
+    person = (
+        await DB.PersonResolver()
+        .fields_to_return({*DB.Person.__fields__.keys(), "full_name"})
+        .get(slug="jeremy")
+    )
+    print(person.full_name)
+
+    person = await DB.PersonResolver().include_fields({"full_name"}).get(slug="jeremy")
+    print(person.full_name)
+
+    person = await DB.PersonResolver().include_computed_fields().get(slug="jeremy")
+    print(person.full_name)
+
+
+async def main():
+    await test_get()
+    await test_nested()
+    await test_add()
+    await raw_add_test()
+    await test_raw_add_many()
+    await test_raw_insert_without_for_loop()
+    await test_delete()
+    await test_update()
+    await test_link()
+    await test_update_links()
+    await test_batch()
+    await test_refresh()
+    await test_transaction()
+    await test_upsert()
+
+    await test_generated_add()
+    await test_generated_delete()
+
+    await test_extra_fields()
+    await test_bulk_reads()
+    await investigate_bulk_load_bug()
     await using_for()
+    await test_add_many_new()
+    await creating_strings_testing()
+    await testing_with_dog()
+    await testing_with_dog_raw()
+    await testing_with_person()
+    await testing_add_many_with_people()
+
+    await test_default_computed_fields()
 
     # expensive ones
     # await add_lots_async()
